import React, { useContext, useEffect, useState } from 'react';
import { useHistory, matchPath, useLocation } from 'react-router-dom';

var context = React.createContext({
  mounted: {},
  setMounted: function setMounted() {
    return undefined;
  },
  getMount: function getMount() {
    return false;
  },
  doMount: function doMount() {
    return undefined;
  },
  doUnmount: function doUnmount() {
    return undefined;
  }
});
context.displayName = 'RouterEx';

var useRouteMount = function useRouteMount() {
  var _useContext = useContext(context),
      mounted = _useContext.mounted,
      setMounted = _useContext.setMounted,
      getMount = _useContext.getMount,
      doMount = _useContext.doMount,
      doUnmount = _useContext.doUnmount;

  return {
    mounted: mounted,
    setMounted: setMounted,
    getMount: getMount,
    doMount: doMount,
    doUnmount: doUnmount
  };
};

function isEmptyChildren(children) {
    return React.Children.count(children) === 0;
}
const Route = (props) => {
    let { computedMatch, location, path, children, component, render } = props;
    const history = useHistory();
    const { getMount, doMount } = useRouteMount();
    const mount = getMount(path);
    let match = null;
    if (computedMatch) {
        match = computedMatch;
    }
    else if (path && location?.pathname) {
        match = matchPath(location.pathname, props);
    }
    const newProps = {
        history,
        location,
        match,
    };
    if (Array.isArray(children) && isEmptyChildren(children)) {
        children = null;
    }
    const renderMatch = (mount) => {
        if (mount) {
            if (children) {
                // @ts-ignore
                return typeof children === 'function' ? children(newProps) : children;
            }
            else if (component) {
                return React.createElement(component, newProps);
            }
            else if (render) {
                // @ts-ignore
                return render(newProps);
            }
        }
        return null;
    };
    useEffect(() => {
        if (!mount && match && path) {
            doMount(path);
        }
    }, [mount, match, doMount, path]);
    return React.createElement(React.Fragment, null, renderMatch(mount));
};

const Router = (props) => {
    const { children } = props;
    const [mounted, setMounted] = useState({});
    const getMount = (path) => {
        if (!path) {
            return false;
        }
        return mounted[path];
    };
    const updateMount = (path, mount) => {
        if (typeof path === 'string') {
            setMounted({
                ...mounted,
                [path]: mount,
            });
        }
        else if (Array.isArray(path)) {
            setMounted({
                ...mounted,
                ...path.reduce((accumulate, current) => {
                    accumulate[current] = mount;
                    return accumulate;
                }, {}),
            });
        }
    };
    const doMount = (path) => {
        updateMount(path, true);
    };
    const doUnmount = (path) => {
        updateMount(path, false);
    };
    const provider = {
        mounted,
        setMounted,
        getMount,
        doMount,
        doUnmount,
    };
    return (React.createElement(context.Provider, { value: provider }, children));
};

const Switch = (props) => {
    const { children, onChange } = props;
    const location = useLocation();
    let isMatch = false;
    return (React.createElement(React.Fragment, null, React.Children.map(children, (child) => {
        if (React.isValidElement(child)) {
            let visible = false;
            let match;
            if (!isMatch) {
                const path = child.props.path || child.props.from;
                match = path
                    ? matchPath(location.pathname, { ...child.props, path })
                    : undefined;
                if (match) {
                    isMatch = true;
                    visible = true;
                    if (typeof onChange === 'function' && match.path !== '*') {
                        onChange(location);
                    }
                }
            }
            return (React.createElement("div", { id: child.props.path, style: { display: visible ? 'block' : 'none' } }, React.cloneElement(child, { location, computedMatch: match })));
        }
        return null;
    })));
};

export { Route, Router, context as RouterContext, Switch, useRouteMount };
